/*
 * An object for a "host" directory provided via the C library.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <thread>

#include <dirent.h>

#include <fsclient/client.h>

#include "host_directory.h"



/* Thread payload. */

static void read_directory(const char *path, file_t *writer)
{
    DIR *dir = opendir(path);
    struct dirent *dirent;

    /* Write directory entries to the pipe, closing the pipe when finished. */

    while ((dirent = readdir(dir)) != NULL)
    {
        offset_t nwritten = client_write(writer, (const void *) dirent, dirent->d_reclen);

        /* Stop writing if the pipe is closed. */

        if (nwritten < dirent->d_reclen)
            break;
    }

    client_close(writer);
}



HostDirectory::~HostDirectory()
{
}

/* Open a directory for reading directory entries. */

long HostDirectory::_opendir(flags_t flags, offset_t *size, l4_cap_idx_t *cap,
                             object_flags_t *object_flags)
{
    file_t *reader, *writer;

    /* Mapping of the reader's memory region should be avoided because no use
       of the reader will be made here. */

    long err = client_pipe(&reader, &writer, 0);

    if (err)
        return err;

    *size = reader->size;
    *cap = reader->ref;
    *object_flags = 0;      /* does not support mmap, has no fixed size */

    /* Discard the reader structure but preserve the capability. */

    reader->ref = L4_INVALID_CAP;
    file_close(reader);

    /* Spawn a independent thread for reading the directory details and writing
       them to the pipe. */

    std::thread(read_directory, path, writer).detach();

    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
